/*
 * Copyright (C) Sapphirecode - All Rights Reserved
 * This file is part of Snippeteer which is released under BSD-3-Clause.
 * See file 'LICENSE' for full license details.
 * Created by Timo Hocker <timo@scode.ovh>, March 2020
 */

/* eslint-disable no-sync */
/* eslint-disable no-console */

'use strict';

const fs = require ('fs-extra');
const path = require ('path');

/**
 * copies the full template to a new folder named after arg[0]
 *
 * @param {string} folder folder to run in
 * @param {Array} args function arguments
 */
async function run (folder, args) {
  const template = path.join (
    __dirname,
    args[0] === 'csv' ? 'csv.template.js' : 'template.js'
  );
  const target_name = args[0] === 'csv' ? '000001_csv_ingress' : args[0];
  const dest = path.join (folder, `${target_name}.js`);
  await fs.writeFile (dest, await fs.readFile (template));
}

/**
 * checks if the arguments meet the requirements
 *
 * @param {string} folder folder to run in
 * @param {Array} args function arguments
 * @returns {boolean} true if arguments match requirements
 */
function assert (folder, args) {
  const tests = [
    {
      f:      () => (args.length < 2),
      reason: 'too many arguments'
    },
    {
      f:      () => (args.length === 1 && typeof args[0] === 'string'),
      reason: 'name is not a string'
    },
    {
      f:      () => ((/^(?:[0-9]+[-_][a-z-_]+|csv)$/iu).test (args[0])),
      reason: 'name has to match /^[0-9]+[-_][a-z-_]+$/'
    },
    {
      f:      () => (typeof folder === 'string'),
      reason: 'cwd is not a folder (internal error)'
    },
    {
      f:      () => (fs.existsSync (folder)),
      reason: 'cwd does not exist (internal error)'
    },
    {
      f:      () => (!fs.existsSync (path.join (folder, args[0]))),
      reason: 'migration already exists'
    }
  ];
  for (const test of tests) {
    if (!test.f ()) {
      console.log (test.reason);
      return false;
    }
  }

  return true;
}

module.exports = { run, assert };
